function [] = visualize_mapping(filename,style)	

	data = dlmread(filename,',',1,0);

	sData = size(data);
	nbHits = sData(1)/(4*4*6); % number of hits per input (sharpness,contrast,brush)
	
	%--- gather the data, for each input sharpness, contrast and brush size ---%
	% data
	data_sharpness = zeros(4,4,6,nbHits);
	data_contrast = zeros(4,4,6,nbHits);
	ind = ones(4,4,6);
	
	% fitting matrices
	fit_order = 1;
	A = zeros(sData(1)/6,(fit_order+1)*(fit_order+2)/2,6);
	xc = zeros(sData(1)/6,6);
	xd = zeros(sData(1)/6,6);
	ind_fit = [0 0 0 0 0 0];
	
	for(i=1:sData(1))
		if(data(i,4) == 3 | data(i,4) == 6)
		  brush_indice = data(i,4)/3+1;
		else
		  brush_indice = data(i,4)/4+1;
		end
		
		true_sharpness = data(i,2);
		true_d = 1-true_sharpness;
		true_indice_s = (round((true_sharpness-0.044)/0.017))/3+1;
		
		true_contrast = data(i,3);
		true_c = (true_contrast+0.1)^(1/3)-(0.1)^(1/3);
		true_indice_c = (round((true_c-0.046)/0.031))/2+1;
		
		user_sharpness = data(i,5);
		user_d = 1-user_sharpness;
		user_contrast = data(i,6);
		user_c = (user_contrast+0.1).^(1/3) - (0.1)^(1/3);
		
		ind_fit(brush_indice) = ind_fit(brush_indice)+1;
		ind_poly = 0;
		for(m=1:fit_order+1)
			for(n=1:fit_order+1-(m-1))
				ind_poly = ind_poly+1;
				A(ind_fit(brush_indice),ind_poly, brush_indice) = (true_c^(m-1)) * (true_d^(n-1)) ;
			end
		end
		xc(ind_fit(brush_indice),brush_indice) = user_c;%-true_c;
		xd(ind_fit(brush_indice),brush_indice) = user_d;%-true_d;
		
		data_sharpness(true_indice_s,true_indice_c,brush_indice,ind(true_indice_s,true_indice_c,brush_indice)) = user_sharpness;
		data_contrast(true_indice_s,true_indice_c,brush_indice,ind(true_indice_s,true_indice_c,brush_indice)) = user_contrast;
		ind(true_indice_s,true_indice_c,brush_indice) = ind(true_indice_s,true_indice_c,brush_indice)+1;
	end
	
	for(s=1:6)
		A_tmp(:,:) = A(:,:,s);
		xc_tmp = xc(:,s);
		xd_tmp = xd(:,s);
		mapping_c(:,s) = A_tmp\xc_tmp;
		mapping_d(:,s) = A_tmp\xd_tmp;
    end
    
    %remove the realistic component
    %for(s=6:-1:1)
%		mapping_c(:,s) = mapping_c(:,s)-mapping_c(:,1);
%        mapping_c(3,s) = 1+mapping_c(3,s);
%		mapping_d(:,s) = mapping_d(:,s)-mapping_d(:,1);
%        mapping_d(2,s) = 1+mapping_d(2,s);
%   end
	
	d_origin = 1-0.231;  % iso-gloss computed as the distance to the origin of our data (c=0.015, d=0.769)
	c_origin = 0.015;
	
	%----- Dense visualization of the mapping ---
	for(s=1:5) %for each brush size
		figMap = figure();
		%--- iso-lines ---%
		D = 0.1;
		c = 0.0:0.01:0.1;
		d = sqrt( (D.*D-c.*c)./(1.78*1.78) );
		plot(d+d_origin,c+c_origin,'--k');hold on;
		D = 0.2;
		c = 0.0:0.01:0.2;
		d = sqrt( (D.*D-c.*c)./(1.78*1.78) );
		plot(d+d_origin,c+c_origin,'--k');hold on;
		D = 0.3;
		c = 0.0:0.01:0.3;
		d = sqrt( (D.*D-c.*c)./(1.78*1.78) );
		plot(d+d_origin,c+c_origin,'--k');hold on;
		D = 0.4;
		c = 0.0:0.01:0.4;
		d = sqrt( (D.*D-c.*c)./(1.78*1.78) );
		plot(d+d_origin,c+c_origin,'--k');hold on;		
		
		for(c = 0.015:0.04:0.285) %contrast
			for(d = 0.015:0.04:0.285) %sharpness
				c_curr = c + c_origin;
				d_curr = d + d_origin;
			
				%draw an arrow from the true sharpness/contrast to the predicted sharpness/contrast
				c_predicted = 0;
				d_predicted = 0;
				
				c_predicted = mapping_c(3,s)*c_curr + mapping_c(2,s)*d_curr + mapping_c(1,s);
				d_predicted = mapping_d(3,s)*c_curr + mapping_d(2,s)*d_curr + mapping_d(1,s);
				
				diff_sharpness = d_predicted-d_curr;
				diff_contrast = c_predicted-c_curr;
               
				quiver(d+d_origin,c+c_origin,diff_sharpness,diff_contrast,0,'LineWidth',1,'Color',[1 0 0],'MaxHeadSize',0.3);hold on;
				
			end
		end
		
		sdisp = sprintf('c Mapping coeff for brush %d: %f,%f,%f',(s-1)*4,mapping_c(3,s),mapping_c(2,s),mapping_c(1,s));
		%sdisp
		sdisp = sprintf('d Mapping coeff for brush %d: %f,%f,%f',(s-1)*4,mapping_d(3,s),mapping_d(2,s),mapping_d(1,s));
		%sdisp
		
		%--- axis and label ---%
		axis equal;
		%set(gca,'DataAspectRatio',[1 1.7 1])
		axis([d_origin d_origin+0.25 c_origin c_origin+0.25]);
		xlabel('Perceptual sharpness d', 'FontSize', 28);
		ylabel('Perceptual contrast c', 'FontSize', 28);
		set(gca,'FontSize',24);
		
		filename = sprintf(strcat('../data_lab/plots/fitting/',style,'_fitting_withrealistic_%d.pdf'),(s-1)*4)
		%saveas(figMap,filename);

	end
	
	
	%----- Dense visualization of the inverse mapping ---
	for(s=1:5) %for each brush size
		figMap = figure();
		%--- iso-lines ---%
		D = 0.1;
		c = 0.0:0.01:0.1;
		d = sqrt( (D.*D-c.*c)./(1.78*1.78) );
		plot(d+d_origin,c+c_origin,'--k');hold on;
		D = 0.2;
		c = 0.0:0.01:0.2;
		d = sqrt( (D.*D-c.*c)./(1.78*1.78) );
		plot(d+d_origin,c+c_origin,'--k');hold on;
		D = 0.3;
		c = 0.0:0.01:0.3;
		d = sqrt( (D.*D-c.*c)./(1.78*1.78) );
		plot(d+d_origin,c+c_origin,'--k');hold on;
		D = 0.4;
		c = 0.0:0.01:0.4;
		d = sqrt( (D.*D-c.*c)./(1.78*1.78) );
		plot(d+d_origin,c+c_origin,'--k');hold on;	
		
		for(c = 0.015:0.04:0.285) %contrast
			for(d = 0.015:0.04:0.285) %sharpness
				c_curr = c + c_origin;
				d_curr = d + d_origin;
			
				%draw an arrow from the true sharpness/contrast to the predicted sharpness/contrast
				c_predicted = 0;
				d_predicted = 0;
				
				c_predicted = ( mapping_d(2,s)*c_curr - mapping_c(2,s)*d_curr - mapping_d(2,s)*mapping_c(1,s) + mapping_c(2,s)*mapping_d(1,s) )/...
										( mapping_d(2,s)*mapping_c(3,s) - mapping_c(2,s)*mapping_d(3,s) );
					
										
				d_predicted = ( mapping_d(3,s)*c_curr - mapping_c(3,s)*d_curr - mapping_d(3,s)*mapping_c(1,s) + mapping_c(3,s)*mapping_d(1,s) )/...
									( mapping_d(3,s)*mapping_c(2,s) - mapping_c(3,s)*mapping_d(2,s) );
				
				diff_sharpness = d_predicted-d_curr;
				diff_contrast = c_predicted-c_curr;
				
				quiver(d+d_origin,c+c_origin,diff_sharpness,diff_contrast,0,'LineWidth',1,'Color',[0 1 0],'MaxHeadSize',0.3);hold on;
				
			end
		end
		
		%--- axis and label ---%
		axis equal;
		%set(gca,'DataAspectRatio',[1 1.7 1])
		axis([d_origin d_origin+0.25 c_origin c_origin+0.25]);
		xlabel('Perceptual sharpness d', 'FontSize', 28);
		ylabel('Perceptual contrast c', 'FontSize', 28);
		set(gca,'FontSize',24);
		
		filename = sprintf(strcat('../data_lab/plots/fitting/',style,'_fitting_inverse_withrealistic_%d.pdf'),(s-1)*4)
		%saveas(figMap,filename);

	end